dockspawn.Dialog = function(panel, dockManager)
{
    this.panel = panel;
    this.zIndexCounter = 1000;
    this.dockManager = dockManager;
    this.eventListener = dockManager;
    this._initialize();
};

dockspawn.Dialog.fromElement = function(id, dockManager)
{
    return new dockspawn.Dialog(new dockspawn.PanelContainer(document.getElementById(id), dockManager), dockManager);
};

dockspawn.Dialog.prototype._initialize = function()
{
    this.panel.floatingDialog = this;
    this.elementDialog = document.createElement('div');
    this.elementDialog.appendChild(this.panel.elementPanel);
    this.draggable = new dockspawn.DraggableContainer(this, this.panel, this.elementDialog, this.panel.elementTitle);
    this.resizable = new dockspawn.ResizableContainer(this, this.draggable, this.draggable.topLevelElement);

    document.body.appendChild(this.elementDialog);
    this.elementDialog.classList.add("dialog-floating");
    this.elementDialog.classList.add("rounded-corner-top");
    this.panel.elementTitle.classList.add("rounded-corner-top");

    this.mouseDownHandler = new dockspawn.EventHandler(this.elementDialog, 'mousedown', this.onMouseDown.bind(this));
    this.resize(this.panel.elementPanel.clientWidth, this.panel.elementPanel.clientHeight);
    this.bringToFront();
};

dockspawn.Dialog.prototype.setPosition = function(x, y)
{
    this.elementDialog.style.left = x + "px";
    this.elementDialog.style.top = y + "px";
};

dockspawn.Dialog.prototype.onMouseDown = function(e)
{
    this.bringToFront();
};

dockspawn.Dialog.prototype.destroy = function()
{
    if (this.mouseDownHandler)
    {
        this.mouseDownHandler.cancel();
        delete this.mouseDownHandler;
    }
    this.elementDialog.classList.remove("rounded-corner-top");
    this.panel.elementTitle.classList.remove("rounded-corner-top");
    removeNode(this.elementDialog);
    this.draggable.removeDecorator();
    removeNode(this.panel.elementPanel);
    delete this.panel.floatingDialog;
};

dockspawn.Dialog.prototype.resize = function(width, height)
{
    this.resizable.resize(width, height);
};

dockspawn.Dialog.prototype.setTitle = function(title)
{
    this.panel.setTitle(title);
};

dockspawn.Dialog.prototype.setTitleIcon = function(iconName)
{
    this.panel.setTitleIcon(iconName);
};

dockspawn.Dialog.prototype.bringToFront = function()
{
    this.elementDialog.style.zIndex = this.zIndexCounter++;
};
